@@ -7,32 +7,62 @@ Coro::ProcessPool - an asynchronous process pool
 =head1 SYNOPSIS
 
     use Coro::ProcessPool;
+    use Coro;
 
     my $pool = Coro::ProcessPool->new(
         max_procs => 4,
         max_reqs  => 100,
+        include   => ['/path/to/my/task/classes', '/path/to/other/packages'],
     );
 
     my $double = sub { $_[0] * 2 };
 
+    #-----------------------------------------------------------------------
     # Process in sequence
+    #-----------------------------------------------------------------------
     my %result;
     foreach my $i (1 .. 1000) {
         $result{$i} = $pool->process($double, [$i]);
     }
 
+    #-----------------------------------------------------------------------
     # Process as a batch
+    #-----------------------------------------------------------------------
     my @results = $pool->map($double, 1 .. 1000);
 
+    #-----------------------------------------------------------------------
     # Defer waiting for result
+    #-----------------------------------------------------------------------
     my %deferred = map { $_ => $pool->defer($double, [$_]) } 1 .. 1000);
     foreach my $i (keys %deferred) {
         print "$i = " . $deferred{$i}->() . "\n";
     }
 
+    #-----------------------------------------------------------------------
     # Use a "task class", implementing 'new' and 'run'
+    #-----------------------------------------------------------------------
     my $result = $pool->process('Task::Doubler', 21);
 
+    #-----------------------------------------------------------------------
+    # Pipelines (work queues)
+    #-----------------------------------------------------------------------
+    my $pipe = $pool->pipeline;
+
+    # Start producer thread to queue tasks
+    my $producer = async {
+        while (my $task = get_next_task()) {
+            $pipe->queue('Some::TaskClass', $task);
+        }
+
+        # Let the pipeline know no more tasks are coming
+        $pipe->shutdown;
+    };
+
+    # Collect the results of each task as they are received
+    while (my $result = $pipe->next) {
+        do_stuff_with($result);
+    }
+
     $pool->shutdown;
 
 =head1 DESCRIPTION
@@ -54,7 +84,7 @@ use Coro::ProcessPool::Util;
 use Coro::Semaphore;
 require Coro::ProcessPool::Pipeline;
 
-our $VERSION = '0.25';
+our $VERSION = '0.26';
 
 if ($^O eq 'MSWin32') {
     die 'MSWin32 is not supported';
@@ -89,6 +119,19 @@ has max_reqs => (
     default => sub { 0 },
 );
 
+=head2 include
+
+An optional array ref of directory paths to prepend to the set of directories
+the worker process will use to find Perl packages.
+
+=cut
+
+has include => (
+    is      => 'ro',
+    isa     => ArrayRef[Str],
+    default => sub { [] },
+);
+
 =head1 PRIVATE ATTRIBUTES
 
 =head2 procs_lock
@@ -122,16 +165,26 @@ has num_procs => (
 
 =head2 procs
 
-Array holding the L<Coro::Process> objects.
+Array holding the L<Coro::ProcessPool::Process> objects.
 
 =cut
 
 has procs => (
     is      => 'ro',
-    isa     => ArrayRef[InstanceOf['Coro::Process']],
+    isa     => ArrayRef[InstanceOf['Coro::ProcessPool::Process']],
     default => sub { [] },
 );
 
+=head2 all_procs
+
+=cut
+
+has all_procs => (
+    is      => 'ro',
+    isa     => HashRef[InstanceOf['Coro::ProcessPool::Process']],
+    default => sub { {} },
+);
+
 =head2 is_running
 
 Boolean which signals to the instance that the C<shutdown> method has been
@@ -151,36 +204,54 @@ has is_running => (
 
 sub DEMOLISH {
     my $self = shift;
-    $self->shutdown;
+    if ($self->is_running) {
+        $self->shutdown;
+    }
 }
 
 sub BUILD {
     my $self = shift;
     for (1 .. $self->max_procs) {
-      $self->checkin_proc($self->start_proc);
+        unshift @{$self->procs}, $self->start_proc;
     }
 }
 
 sub start_proc {
     my $self = shift;
-    my $proc = Coro::ProcessPool::Process->new();
+    my $proc = Coro::ProcessPool::Process->new(include => $self->include);
+    my $pid  = $proc->pid;
     ++$self->{num_procs};
+    $self->{all_procs}{$pid} = $proc;
     return $proc;
 }
 
 sub kill_proc {
     my ($self, $proc) = @_;
+    my $pid  = $proc->pid;
     $proc->shutdown;
     --$self->{num_procs};
+    delete $self->{all_procs}{$pid};
 }
 
 sub checkin_proc {
     my ($self, $proc) = @_;
 
-    if (!$self->is_running || ($self->max_reqs && $proc->messages_sent >= $self->max_reqs)) {
+    unless ($self->is_running) {
+      $self->kill_proc($proc);
+      return;
+    }
+
+    if (!$proc->is_running) {
+        my $pid = $proc->pid;
+        --$self->{num_procs};
+        delete $self->{all_procs}{$pid};
+        unshift @{$self->procs}, $self->start_proc;
+    }
+    elsif ($self->max_reqs && $proc->messages_sent >= $self->max_reqs) {
         $self->kill_proc($proc);
         unshift @{$self->procs}, $self->start_proc;
-    } else {
+    }
+    else {
         unshift @{$self->procs}, $proc;
     }
 }
@@ -223,13 +294,14 @@ sub shutdown {
     my $self = shift;
 
     $self->is_running(0);
+    $_->shutdown(5) foreach values %{$self->{all_procs}};
 
-    my $count = $self->num_procs or return;
-    for (1 .. $count) {
-        my $proc = shift @{$self->procs};
-        $proc->shutdown(5);
-        --$self->{num_procs};
-    }
+    $self->{procs}      = [];
+    $self->{all_proces} = {};
+    $self->{num_procs}  = 0;
+    $self->{procs_lock} = Coro::Semaphore->new($self->max_procs);
+
+    return;
 }
 
 =head2 process($f, $args)
@@ -304,8 +376,8 @@ sub defer {
 
 Returns a L<Coro::ProcessPool::Pipeline> object which can be used to pipe
 requests through to the process pool. Results then come out the other end of
-the pipe.  It is up to the calling code to perform task account (for example,
-by passing an id in as one of the arguments to the task class).
+the pipe. It is up to the calling code to perform task account (for example, by
+passing an id in as one of the arguments to the task class).
 
     my $pipe = $pool->pipeline;
 
@@ -325,6 +397,10 @@ All arguments to C<pipeline()> are passed transparently to the constructor of
 L<Coro::ProcessPool::Pipeline>. There is no limit to the number of pipelines
 which may be created for a pool.
 
+If the pool is shutdown while the pipeline is active, any tasks pending in
+L<Coro::ProcessPool::Pipeline::next> will fail and cause the next call to
+C<next()> to croak.
+
 =cut
 
 sub pipeline {
@@ -442,9 +518,20 @@ process pool on Windows:
 
 Jeff Ober <jeffober@gmail.com>
 
-=head1 LICENSE
+=head1 COPYRIGHT AND LICENSE
 
-BSD License
+Copyright (C) 2013-2015, Jeff Ober.
+All rights reserved.
+
+Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
+
+1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
+
+2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
+
+3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 =cut
 
