@@ -105,7 +105,6 @@ has complete => (
     isa      => InstanceOf['Coro::Channel'],
     init_arg => undef,
     default  => sub { Coro::Channel->new() },
-    handles  => { next => 'get' }
 );
 
 =head1 METHODS
@@ -115,6 +114,19 @@ has complete => (
 Cedes control until a previously queued task is complete and the result is
 available.
 
+=cut
+
+sub next {
+    my $self = shift;
+    my $finished = $self->complete->get or return;
+    my ($result, $error) = @$finished;
+    if ($error) {
+        croak $error;
+    } else {
+        return $result;
+    }
+}
+
 =head2 queue($task, $args)
 
 Queues a new task. Arguments are identical to L<Coro::ProcessPool::process> and
@@ -124,22 +136,31 @@ L<Coro::ProcessPool::defer>.
 
 sub queue {
     my ($self, @args) = @_;
-    croak 'pipeline is shut down' if $self->{is_shutdown};
-    croak 'pipeline is shutting down' if $self->{shutting_down};
+    croak 'pipeline is shut down' if $self->is_shutdown;
+    croak 'pipeline is shutting down' if $self->shutting_down;
+
+    unless ($self->pool->is_running) {
+        unless ($self->is_shutdown || $self->shutting_down) {
+            $self->shutdown;
+        }
+
+        croak 'pool is not running';
+    }
 
     my $deferred = $self->pool->defer(@args);
 
     async_pool {
         my ($self, $deferred) = @_;
-        my $result = $deferred->();
+        my $result = eval { $deferred->() };
 
-        $self->{complete}->put($result);
+        $self->complete->put([$result, $@]);
         --$self->{num_pending};
 
-        if ($self->{num_pending} == 0) {
-            if ($self->{shutting_down} || $self->{auto_shutdown}) {
-                $self->{complete}->shutdown;
-                $self->{is_shutdown} = 1;
+        if ($self->num_pending == 0) {
+            if ($self->shutting_down || $self->auto_shutdown) {
+                $self->complete->shutdown;
+                $self->is_shutdown(1);
+                $self->shutting_down(0);
             }
         }
     } $self, $deferred;
@@ -155,7 +176,7 @@ Signals shutdown of the pipeline. A shutdown pipeline may not be reused.
 
 sub shutdown {
     my $self = shift;
-    $self->{shutting_down} = 1;
+    $self->shutting_down(1);
 }
 
 1;
