@@ -1,6 +1,7 @@
 package Coro::ProcessPool::Process;
 
 use Moo;
+use AnyEvent;
 use Carp;
 use Coro;
 use Coro::AnyEvent;
@@ -25,14 +26,25 @@ BEGIN {
 
 sub BUILDARGS {
     my ($class, %args) = @_;
+    my $include = delete $args{include};
+
+    $include = [] unless defined $include;
+    $include = [$include] unless ref $include;
 
     my ($r, $w, $e) = (gensym, gensym, gensym);
     my $cmd  = get_command_path;
-    my $args = get_args;
+    my $args = get_args(@$include);
     my $exec = "$cmd $args";
     my $pid  = open3($w, $r, $e, $exec) or croak "Error spawning process: $!";
 
+    # Get real child process id (open3 creates parent -> sh -> perl). Worker
+    # will send its own pid once initialized.
+    my $child = <$r>;
+    $child or croak 'unable to talk to child process';
+    chomp $child;
+
     $args{pid}       = $pid;
+    $args{child}     = $child;
     $args{child_in}  = unblock $r;
     $args{child_out} = unblock $w;
     $args{child_err} = unblock $e;
@@ -52,6 +64,29 @@ sub DEMOLISH {
     $self->shutdown;
 }
 
+has watcher => (
+    is       => 'lazy',
+    isa      => InstanceOf['Coro'],
+    init_arg => undef,
+);
+
+sub _build_watcher {
+    my $self = shift;
+
+    async {
+        my $rouse_cb = Coro::rouse_cb;
+        my $event = AnyEvent->child(pid => $self->child, cb => $rouse_cb);
+        my ($pid, $code) = Coro::rouse_wait($rouse_cb);
+
+        if ($code != 0) {
+            $self->join;
+            $self->cleanup;
+        }
+
+        return;
+    };
+}
+
 has messages_sent => (
     is       => 'rw',
     isa      => Int,
@@ -59,12 +94,23 @@ has messages_sent => (
     default  => sub { 0 },
 );
 
+has is_running => (
+    is       => 'rw',
+    isa      => Bool,
+    init_arg => undef,
+    default  => sub { 1 },
+);
+
 has pid => (
     is        => 'ro',
     isa       => Int,
     required  => 1,
-    clearer   => 'clear_pid',
-    predicate => 'is_running',
+);
+
+has child => (
+    is        => 'ro',
+    isa       => Int,
+    required  => 1,
 );
 
 has child_in => (
@@ -114,6 +160,8 @@ sub _build_child_in_watcher {
                 warn "Unexpected message received: $id";
             }
         }
+
+        $self->cleanup if $self->is_running;
     } @_;
 }
 
@@ -131,6 +179,8 @@ sub _build_child_err_watcher {
             warn sprintf("(WORKER PID %s) %s", ($self->pid || '(DEAD)'), $line);
             last unless $self->pid;
         }
+
+        $self->cleanup if $self->is_running;
     } @_;
 }
 
@@ -143,18 +193,24 @@ sub cleanup {
     }
 
     if ($self->child_in) {
-        $self->child_in_watcher->join;
+        $self->child_in_watcher->safe_cancel;
         $self->child_in->close;
         $self->clear_child_in_watcher;
         $self->clear_child_in;
     }
 
     if ($self->child_err) {
-        $self->child_err_watcher->join;
+        $self->child_err_watcher->safe_cancel;
         $self->child_err->close;
         $self->clear_child_err_watcher;
         $self->clear_child_err;
     }
+
+    foreach my $id (keys %{$self->inbox}) {
+        $self->inbox->{$id}->croak('process killed while waiting on this task to complete');
+    }
+
+    $self->is_running(0);
 }
 
 sub join {
@@ -175,25 +231,14 @@ sub join {
         }
     }
 
-    $self->clear_pid;
     return 1;
 }
 
 sub kill_process {
     my ($self, $timeout) = @_;
-
     return unless $self->is_running;
-
-    local $SIG{CHLD} = 'IGNORE';
-
-    my $id = $self->write('SHUTDOWN');
-    my $reply = $self->recv($id);
-    $self->child_out->close;
-
-    until ($self->join($timeout)) {
-        kill('KILL', $self->pid);
-        waitpid($self->pid, 0);
-    }
+    kill('KILL', $self->child);
+    waitpid($self->pid, 0);
 }
 
 sub shutdown {
@@ -206,6 +251,8 @@ sub shutdown {
 
 sub write {
     my ($self, $task, $args) = @_;
+    croak 'not running' unless $self->is_running;
+    croak 'process disconnected' unless $self->child_out;
     my $id = Data::UUID->new->create_str();
     $self->inbox->{$id} = AnyEvent->condvar;
     $self->child_out->print(encode($id, $task, $args) . $EOL);
@@ -215,9 +262,7 @@ sub write {
 
 sub send {
     my ($self, $f, $args) = @_;
-    croak 'not running' unless $self->is_running;
-    $args ||= [];
-    return $self->write($f, $args);
+    return $self->write($f, $args || []);
 }
 
 sub recv {
